require 'spec_helper'

describe NdSync::Local do
  before(:all) do
    class Model < ActiveRecord::Base
    end

    class View < ActiveRecord::Base
      include NdSync::Local
      attr_accessor :transcribed

      alias :transcribed? :transcribed
    end

    Model.connection.create_table :models do |t|
      t.integer :paso
      t.integer :web_id
    end
    Model.connection.create_table :views do |t|
      t.integer :step
      t.integer :desired_step
      t.string  :origin_table
      t.string  :origin_id_field
      t.string  :origin_step_field
      t.integer :origin_id
    end
  end

  before do
    Model.delete_all
    View.delete_all
  end

  it "makes transcriber available" do
    View.transcriber do
      self.transcribed = true
    end

    expect(View.transcribe(nil)).to be_transcribed
  end

  it "transmits objects where step is desired_step - 1" do
    vi1 = View.create! step: 0, desired_step: 2
    vi2 = View.create! step: 1, desired_step: 2
    vi3 = View.create! step: 2, desired_step: 2

    pendings = View.to_transmit

    expect(pendings     ).to be_kind_of Array
    expect(pendings.size).to eq 1
    expect(pendings     ).not_to include vi1
    expect(pendings     ).to     include vi2
    expect(pendings     ).not_to include vi3
  end

  it "generates model class dinamically" do
    vi = View.new(origin_table: 'models', origin_id_field: 'code')

    expect(vi.model_class.superclass ).to be ActiveRecord::Base
    expect(vi.model_class.table_name ).to eq 'models'
    expect(vi.model_class.primary_key).to eq 'code'
  end

  it "loads it's model's object" do
    vi = View.new(origin_table: 'models', origin_id_field: 'id', origin_id: 3)
    Model.create!(id: 1)
    mo = Model.create!(id: 3)

    expect(vi.model_object.id).to eq mo.id
  end

  describe 'transmitted_to' do
    it "marks model as synched" do
      vi  = View.new(origin_table: 'models', origin_id_field: 'id', origin_id: 3,
        origin_step_field: 'paso', step: 1, desired_step: 2)
      mo1 = Model.create!(id: 3, paso: 1)
      mo2 = Model.create!(id: 1, paso: 1)

      vi.transmitted_to double(id: 5)

      mo1.reload
      mo2.reload
      expect(mo1.paso).to eq 2
      expect(mo2.paso).to eq 1
    end

    context 'view responds to web_id' do
      it "updates its model's web_id" do
        vi  = View.new(origin_table: 'models', origin_id_field: 'id', origin_id: 3,
          origin_step_field: 'paso', step: 1, desired_step: 2)
        def vi.web_id
          nil
        end
        mo1 = Model.create!(id: 3, paso: 1)

        vi.transmitted_to double(id: 5)

        mo1.reload
        expect(mo1.web_id).to eq 5
      end
    end

    context 'received object contains errors' do
      it "marks object as problematic" do
        vi  = View.new(origin_table: 'models', origin_id_field: 'id', origin_id: 3,
          origin_step_field: 'paso', step: 1, desired_step: 2)
        mo1 = Model.create!(id: 3, paso: 1)
        mo2 = Model.create!(id: 1, paso: 1)

        vi.transmitted_to double(id: 5, errors: ['error'])

        mo1.reload
        mo2.reload
        expect(mo1.paso).to eq -1
        expect(mo2.paso).to eq 1
      end
    end
  end
end
